"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var NomnomlPlugin_1;
Object.defineProperty(exports, "__esModule", { value: true });
const converter_1 = require("typedoc/dist/lib/converter");
const components_1 = require("typedoc/dist/lib/converter/components");
const events_1 = require("typedoc/dist/lib/output/events");
/**
 * Nomnoml plugin component.
 */
let NomnomlPlugin = NomnomlPlugin_1 = class NomnomlPlugin extends components_1.ConverterComponent {
    constructor() {
        super(...arguments);
        this.graphMap = new Map();
        /**
         * Regex literal that matches body closing tag.
         */
        this.BODY_CLOSING_TAG = /<\/body>/;
    }
    /**
     * 1. Load nomnoml library.
     * 2. Initialize nomnoml.
     * 3. Close body tag.
     */
    get customScriptsAndBodyClosingTag() {
        const script = Array.from(this.graphMap.entries())
            .map(([id, source]) => `if (document.getElementById("${id}")) {
            nomnoml.draw(document.getElementById("${id}"), ${JSON.stringify(source)});
          }`)
            .join('\n');
        return `
      <script src="https://cdn.jsdelivr.net/npm/dagre/dist/dagre.min.js"></script>
      <script src="https://cdn.jsdelivr.net/npm/nomnoml/dist/nomnoml.min.js"></script>
      <script>${script}</script>
      </body>
    `;
    }
    /**
     * filter logic for Comment exist
     */
    static filterComment(comment) {
        return comment !== undefined && !!comment;
    }
    /**
     * filter logic for CommentTags exist
     */
    static filterCommentTags(tags) {
        return tags !== undefined && !!tags;
    }
    /**
     * return turn when tag's paramName is 'nomnoml'
     */
    static isNomnomlCommentTag(tag) {
        return tag.tagName === 'nomnoml';
    }
    /**
     * get CommentTags for using `@nomnoml` annotation from Context.
     */
    static nomnomlTags(context) {
        return Object.values(context.project.reflections) // get reflection from context
            .map(reflection => reflection.comment) // get Comment from Reflection
            .filter(this.filterComment) // filter only comment exist
            .map(comment => comment.tags) // get CommentTags from Comment
            .filter(this.filterCommentTags) // filter only CommentTags exist
            .reduce((a, b) => a.concat(b), []) // merge all CommentTags
            .filter(this.isNomnomlCommentTag); // filter tag that paramName is 'nomnoml'
    }
    /**
     * The first line of text wraps h4.
     * The other wraps by div classed nomnoml.
     */
    convertCommentTagText(tagText) {
        const texts = tagText.split('\n');
        // take first line
        const title = texts.shift();
        // the other
        const nomnoml = texts.join('\n');
        const id = `nomnoml-${++NomnomlPlugin_1.graphIdCounter}`;
        this.graphMap.set(id, nomnoml);
        return `#### ${title} \n\n <canvas id="${id}"></canvas>`;
    }
    /**
     * Insert custom script before closing body tag.
     */
    convertPageContents(contents) {
        if (this.BODY_CLOSING_TAG.test(contents)) {
            return contents.replace(this.BODY_CLOSING_TAG, this.customScriptsAndBodyClosingTag);
        }
        return contents;
    }
    /**
     * listen to event on initialization
     */
    initialize() {
        this.listenTo(this.owner, {
            [converter_1.Converter.EVENT_RESOLVE_BEGIN]: this.onResolveBegin,
        }).listenTo(this.application.renderer, {
            [events_1.PageEvent.END]: this.onPageEnd,
        });
    }
    /**
     * Triggered when the converter begins converting a project.
     */
    onResolveBegin(context) {
        NomnomlPlugin_1.nomnomlTags(context).forEach(tag => {
            // convert
            tag.text = this.convertCommentTagText(tag.text);
        });
    }
    /**
     * Triggered after a document has been rendered, just before it is written to disc.
     * Remove duplicate lines to tidy up output
     */
    onPageEnd(page) {
        if (page.contents !== undefined) {
            // convert
            page.contents = this.convertPageContents(page.contents);
        }
    }
};
NomnomlPlugin.graphIdCounter = 0;
NomnomlPlugin = NomnomlPlugin_1 = __decorate([
    components_1.Component({ name: 'nomnoml' })
], NomnomlPlugin);
exports.NomnomlPlugin = NomnomlPlugin;
